/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

/* 
 * File:   StubLtc2600.cpp
 * Author: claude
 * 
 * Created on 20 décembre 2016, 11:25
 */

#include "StubLtc2600.h"

/**
 * Constructeur de la classe d'émulation du Ltc2600.
 * 
 * La méthode s'assure que l'IHM du simulateur de Ltc2600 est en cours d'exécution. 
 */
StubLtc2600::StubLtc2600() {
    FILE *fp;
    char line[ 4096 ];

    // Tenter de récupérer le PID de l'IHM du simulateur de Ltc2600
    fp = popen("pidof virltc2600", "r");
    if (fp == NULL) {
        printf("Echec d'exécution de la commande : 'pidof'\n");
        exit(1);
    }

    // Lire et mémoriser le PID
    fgets(line, sizeof (line) - 1, fp);
    pid = atoi(line);
    if (pid == 0) {
        printf("!! Le simulateur de DAC n'est pas en cours d'exécution !!\n");
        exit(1);
    }

    pclose(fp);

}

/**
 * Constructeur de copie.
 * 
 * @param orig
 */
StubLtc2600::StubLtc2600(const StubLtc2600& orig) {
}

/**
 * Destructeur.
 */
StubLtc2600::~StubLtc2600() {
}

/**
 * Méthode associée à la commande "Power Down n" du Ltc2600
 * 
 * @param n
 * @return 
 */
int StubLtc2600::powerDownDacReg(dacID_t n) {
    unsigned int msg = 0;

    msg = (static_cast<int> (dacCmd_t::POWER_DOWN_N) << 20)
            | (static_cast<int> (n) << 16)
            ;

    puts("\nPOWER_DOWN_N");
    sendFrame(msg);

    return 0;
}

/**
 * Méthode associée à la commande "Update (Power Up) DAC Register n" du Ltc2600
 * 
 * @param n
 * @return 
 */
int StubLtc2600::powerUpDacReg(dacID_t n) {
    unsigned int msg = 0;

    msg = (static_cast<int> (dacCmd_t::UPDATE_DAC_REGISTER_N) << 20)
            | (static_cast<int> (n) << 16)
            ;

    puts("\nUPDATE_DAC_REGISTER_N");
    sendFrame(msg);

    return 0;
}

/**
 * Méthode associée aux commandes "Write to and Update (Power Up) n" et 
 * "Write to Input Register n, Update (Power Up) All n" du Ltc2600
 * @param n Identifiant du/des DAC(s) [DAC_A...DAC_H, DAC_ALL]
 * @param value Valeur 16 bits à écrire dans le "Input Register" et le
 *              "DAC Register" du/des DAC(s)
 * @param shouldUpdateAll Booléen qui permet de cibler l'une ou l'autre commande
 *                        false -> "Write to and Update (Power Up) n"
 *                        true -> Write to Input Register n, Update (Power Up) All n"
 * @return 
 */
int StubLtc2600::updateDacReg(dacID_t n, unsigned short value, bool shouldUpdateAll) {
    unsigned int msg;
    dacCmd_t cmd;

    cmd = shouldUpdateAll
            ? dacCmd_t::WRITE_TO_INPUT_REGISTER_N_UPDATE_ALL_N
            : dacCmd_t::WRITE_TO_AND_UPDATE_N
            ;

    msg = (static_cast<int> (cmd) << 20) | (static_cast<int> (n) << 16) | value;

    puts(shouldUpdateAll ? "\nWRITE_TO_INPUT_REGISTER_N_UPDATE_ALL_N" : "\nWRITE_TO_AND_UPDATE_N");
    sendFrame(msg);

    return 0;
}

/**
 * Méthode associée à la commande "Write to Input Register n" du Ltc2600.
 * 
 * @param n Identifiant du/des DAC(s) [DAC_A...DAC_H, DAC_ALL]
 * @param value Valeur 16 bits à écrire dans le "Input Register" du/des DAC(s)
 * @return 
 */
int StubLtc2600::writeInputReg(dacID_t n, unsigned short value) {
    unsigned int msg;

    msg = (static_cast<int> (dacCmd_t::WRITE_TO_INPUT_REGISTER_N) << 20)
            | (static_cast<int> (n) << 16)
            | value
            ;

    puts("\nWRITE_TO_INPUT_REGISTER_N");
    sendFrame(msg);

    return 0;
}

/**
 * Méthode qui envoie la trame de 32 bits au simulateur par l'intermédiaire d'un
 *  signal temps réel (-> seul signal qui permet d'envoyer une donnée).
 * @param msg
 * @return 
 */
int StubLtc2600::sendFrame(unsigned int msg) {

    printf(">> Envoi de la trame '%u' (0x%08x) au simulateur de DAC (pid : %d)\n", msg, msg, pid);

    _spiFrame.sival_int = msg;
    sigqueue(pid, SIGRTMIN, _spiFrame);

    return 0;
}